@@ -123,6 +123,8 @@ vsg::ref_ptr<vsg::Object> tile::read(const vsg::Path& filename, vsg::ref_ptr<con
     auto extension = vsg::lowerCaseFileExtension(filename);
     if (extension != ".tile") return {};
 
+    if (!options) return {};
+
     auto tile_info = filename.substr(0, filename.length() - 5);
     if (tile_info == "root")
     {
@@ -163,7 +165,7 @@ vsg::ref_ptr<vsg::Object> tile::read_root(vsg::ref_ptr<const vsg::Options> optio
                 {
                     vsg::ComputeBounds computeBound;
                     tile_node->accept(computeBound);
-                    auto& bb = computeBound.bounds;
+                    const auto& bb = computeBound.bounds;
                     vsg::dsphere bound((bb.min.x + bb.max.x) * 0.5, (bb.min.y + bb.max.y) * 0.5, (bb.min.z + bb.max.z) * 0.5, vsg::length(bb.max - bb.min) * 0.5);
 
                     auto plod = vsg::PagedLOD::create();
@@ -179,6 +181,32 @@ vsg::ref_ptr<vsg::Object> tile::read_root(vsg::ref_ptr<const vsg::Options> optio
         }
     }
 
+    // error handling.
+    if (group->children.empty())
+    {
+        // check to see if we required a protocol like http
+        const auto& filename = settings->imageLayer;
+        auto pos = filename.find("://");
+        if (pos != vsg::Path::npos)
+        {
+            auto protocol = filename.substr(0, pos);
+
+            Features features;
+            if (vsg::getFeatures(options, features))
+            {
+                if (auto itr = features.protocolFeatureMap.find(protocol); itr != features.protocolFeatureMap.end())
+                {
+                    return ReadError::create("vsg::tile::read_root(..) could not data.");
+                }
+            }
+            return ReadError::create("vsg::tile::read_root(..) no support available for protocol.");
+        }
+        else
+        {
+            return ReadError::create("vsg::tile::read_root(..) unable to load file.");
+        }
+    }
+
     uint64_t maxLevel = 20;
     uint64_t estimatedNumOfTilesBelow = 0;
     uint64_t maxNumTilesBelow = 1024;
